/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.Navigation");

(function() {

dojo.require("jazz.util.BodyClickListener");
dojo.require("jazz.util.Array");

dojo.requireLocalization("jazz.util", "NavigationMessages");
var _messages = dojo.i18n.getLocalization("jazz.util", "NavigationMessages");

var _navigateAwayHandlers = new Array();
var _bodyClickObj = null;
var _bodyClickCallbackFunctions = new Array();
var array = jazz.util.Array;

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
jazz.util.Navigation.registerNavigateHandler = function(onBeforeUnloadFunc, bodyClickFunc, priority) {
	/*	summary:
	 * 		register functions to be alerted whenever the user attempts to navigate away
	 * 		from the current URL or hash value. 
	 *	onBeforeUnloadFunc: function that is notified just before page unloads. Function must
	 *		return a message that goes in a standard browser confirm box with the following text:
	 *
	 *			Are you sure you want to navigate away from this page?
	 *
	 *			message
	 *			
	 *			Press OK to continue, or Cancel to stay on the current page.
	 *
	 *		Includes navigations initiated by changing the address in the address
	 *		bar, or visiting a bookmark. Return null to pass through, or a message string to 
	 *		prompt the user.
	 *	bodyClickFunc: function that is notified whenever a link click is intercepted or when promptForVeto
	 *		is called.  This function is responsible for determining whether the navigation will be 
	 *		destructive to uncommitted user input, or whether it's just an ajax context switch.  
	 *		- Return false to allow link
	 *		- Return object to prompt user using standard onbeforeunload type prompt: 
	 *			{message: String, callback: Function that will be called after user is prompted, passes shouldVeto boolean}
	 *		- Return true to cancel link (not recommended without providing some user feedback)
	 *	priority: higher number = higher priority 
	 *	return: handler object.
	 *	
	 *	TODO: Integrate with back button support to get notification on backs that involve hash changes. 
	 */
	if (!onBeforeUnloadFunc && !bodyClickFunc) throw new Error("[jazz.util.Navigation.registerNavigateHandler] No callback functions passed");
	
	if (_navigateAwayHandlers.length == 0) {
		window.onbeforeunload = _handleOnBeforeUnload;	
		_bodyClickObj = jazz.util.BodyClickListener.register(jazz.util.Navigation.promptForVeto, 1000);
	}
	var newHandler = {onBeforeUnloadFunc: onBeforeUnloadFunc, bodyClickFunc: bodyClickFunc, p: priority};
	array.insertByPriority(_navigateAwayHandlers,newHandler);
	return newHandler;
}

jazz.util.Navigation.unregisterNavigateHandler = function(handler) {
	/*	summary:
	 * 		unregister navigate handler
	 * 	handler:  the handler object, returned by the registerNavigateHandler function
	 * 	return: success boolean
	 */
	var success = array.removeByReference(_navigateAwayHandlers, handler);
	if (_navigateAwayHandlers.length == 0) {
		window.onbeforeunload = null;
		jazz.util.BodyClickListener.unregister(_bodyClickObj);
	}
	return success;
}

jazz.util.Navigation.promptForVeto = function(href) {
	/*	summary:
	 * 		For occasions when a script wants to change the URL and would like to check for 
	 * 		navigateHandlers programmatically.  Calling promptForVeto asks all registered 
	 * 		navigateHandlers if they require a prompt (passing the href string as the 
	 * 		first param), prompts the user to see if they want to continue (if necessary), 
	 * 		and returns	a boolean based on their answer. 
	 * 	href:  the href that you want to navigate to programatically.
	 * 	return: shouldVeto boolean.  True = don't navigate, False = go ahead
	 */
	return _fakeOnBeforeUnloadPrompt(href);
}
function _handleOnBeforeUnload (e) {
	if (!e) e = window.event;  // need to fetch the event object explicitly in IE
	var message = _getOnBeforeUnloadMessage();
	if (message !== false) {
		e.returnValue = message;
	}
}
function _fakeOnBeforeUnloadPrompt(href) {
	var urlObj = new dojo._Url(href);
	var winLoc = new dojo._Url(window.location.href);
	if (urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path))) {
		//hash change
		var isHashChange = true;
	} else {
		var isHashChange = false;
	}
	
	var message = _getBodyClickMessage(href, isHashChange);
	var shouldVeto = false;
	if (message === true) 
		shouldVeto = true;
	else if (message !== false) {
		//fake the onBeforeUnload dialog
		if (confirm(dojo.string.substitute(_messages.pageUnload, [ message ]))) {
			if (!isHashChange) window.onbeforeunload = null; //The user has agreed to navigate away, so short circuit the real onBeforeUnload
		} else {
			shouldVeto = true;
		}
	}
	for (var i = 0; i < _navigateAwayHandlers.length; i++) {
		//notify callbacks of decision
		for (var i = 0; i < _bodyClickCallbackFunctions.length; i++) {
			_bodyClickCallbackFunctions[i](shouldVeto);	
		}
	}
	return shouldVeto;
}

function _getBodyClickMessage(href, isHashChange) {
	var messages = new Array();
	_bodyClickCallbackFunctions = new Array();
	for (var i = 0; i < _navigateAwayHandlers.length; i++) {
		var obj = _navigateAwayHandlers[i].bodyClickFunc(href, isHashChange);
		if (obj === false) continue;
		else if (obj === true) return true;
		
		if (obj && obj.message) messages.push(obj.message);
		if (obj && obj.callback) _bodyClickCallbackFunctions.push(obj.callback);
	}
	return _parseMessages(messages);
}

function _getOnBeforeUnloadMessage() {
	var messages = new Array();
	for (var i = 0; i < _navigateAwayHandlers.length; i++) {
		var message = _navigateAwayHandlers[i].onBeforeUnloadFunc(); 
		if (message) messages.push(message);
	}
	return _parseMessages(messages);
}

function _parseMessages(messages) {
	if (messages.length > 0) { // if any messages were returned, format the output and show the prompt
		messages.push(_messages.changesDiscarded);
		var output = "";
		for (var i = 0; i < messages.length; i++) {
			if (messages.length > 2 && i != messages.length - 1) output += "- ";
			output += messages[i];
			if (i == messages.length - 2) {
				if (messages.length == 2) {
					output += " ";
				} else {
					output += "\n\n";
				} 
			} else if (i != messages.length - 1) {
				output += "\n";
			}
		}
		return output;
	}
	return false;
}

})();